@@ -453,6 +453,25 @@ finish_loading_1(BIF_ALIST_1)
 	goto done;
     }
 
+    exceptions = 0;
+    for (i = 0; i < n; i++) {
+        p[i].exception = THE_NON_VALUE;
+        if (p[i].modp->curr.code_hdr && p[i].modp->old.code_hdr) {
+            p[i].exception = am_not_purged;
+            exceptions++;
+        }
+    }
+
+    if (exceptions) {
+        res = exception_list(BIF_P, am_not_purged, p, exceptions);
+        goto done;
+    }
+
+    /*
+     * Now we can load all code. This can't fail.
+     */
+    do_commit = 1;
+
     for (i = 0; i < n; i++) {
 	if (p[i].modp->curr.num_breakpoints > 0 ||
 	    p[i].modp->curr.num_traced_exports > 0 ||
@@ -477,42 +496,24 @@ finish_loading_1(BIF_ALIST_1)
 
     exceptions = 0;
     for (i = 0; i < n; i++) {
-	p[i].exception = THE_NON_VALUE;
-	if (p[i].modp->curr.code_hdr && p[i].modp->old.code_hdr) {
-	    p[i].exception = am_not_purged;
-	    exceptions++;
-	}
+        Eterm mod;
+        Eterm retval;
+
+        erts_refc_inc(&p[i].code->intern.refc, 1);
+        retval = erts_finish_loading(p[i].code, BIF_P, 0, &mod);
+        ASSERT(retval == NIL || retval == am_on_load);
+        if (retval == am_on_load) {
+            p[i].exception = am_on_load;
+            exceptions++;
+        }
     }
 
+    /*
+     * Check whether any module has an on_load_handler.
+     */
+
     if (exceptions) {
-	res = exception_list(BIF_P, am_not_purged, p, exceptions);
-    } else {
-	/*
-	 * Now we can load all code. This can't fail.
-	 */
-
-	exceptions = 0;
-	for (i = 0; i < n; i++) {
-	    Eterm mod;
-	    Eterm retval;
-
-	    erts_refc_inc(&p[i].code->intern.refc, 1);
-	    retval = erts_finish_loading(p[i].code, BIF_P, 0, &mod);
-	    ASSERT(retval == NIL || retval == am_on_load);
-	    if (retval == am_on_load) {
-		p[i].exception = am_on_load;
-		exceptions++;
-	    }
-	}
-
-	/*
-	 * Check whether any module has an on_load_handler.
-	 */
-
-	if (exceptions) {
-	    res = exception_list(BIF_P, am_on_load, p, exceptions);
-	}
-	do_commit = 1;
+        res = exception_list(BIF_P, am_on_load, p, exceptions);
     }
 
 done:
@@ -647,7 +648,7 @@ BIF_RETTYPE erts_internal_check_dirty_process_code_2(BIF_ALIST_2)
 {
     erts_aint32_t state;
     Process *rp;
-    int dirty, busy, reds = 0;
+    int busy, reds = 0;
     Eterm res;
 
     if (BIF_P != erts_dirty_process_signal_handler
@@ -669,13 +670,7 @@ BIF_RETTYPE erts_internal_check_dirty_process_code_2(BIF_ALIST_2)
         BIF_RET(am_false);
 
     state = erts_atomic32_read_nob(&rp->state);
-    dirty = (state & ERTS_PSFLG_DIRTY_RUNNING);
-    /*
-     * Ignore ERTS_PSFLG_DIRTY_RUNNING_SYS (see
-     * comment in erts_execute_dirty_system_task()
-     * in erl_process.c).
-     */
-    if (!dirty)
+    if (!ERTS_PROC_IN_DIRTY_STATE(state))
         BIF_RET(am_normal);
 
     busy = erts_proc_trylock(rp, ERTS_PROC_LOCK_MAIN) == EBUSY;
