@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson 2015-2023. All Rights Reserved.
+ * Copyright Ericsson 2015-2024. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -41,9 +41,17 @@
 #include <stdio.h>
 #include <signal.h>
 #include <locale.h>
-#ifdef HAVE_TERMCAP
+#if defined(HAVE_TERMCAP) && (defined(HAVE_TERMCAP_H) || (defined(HAVE_CURSES_H) && defined(HAVE_TERM_H)))
 #include <termios.h>
+#ifdef HAVE_TERMCAP_H
 #include <termcap.h>
+#else /* !HAVE_TERMCAP_H */
+#include <curses.h>
+#include <term.h>
+#endif
+#else
+/* We detected TERMCAP support, but could not find the correct headers to include */
+#undef HAVE_TERMCAP
 #endif
 #ifndef __WIN32__
 #include <unistd.h>
@@ -336,19 +344,25 @@ static ERL_NIF_TERM tty_write_nif(ErlNifEnv* env, int argc, const ERL_NIF_TERM a
                 res = writev(tty->ofd, iov, iovcnt);
             } while(res < 0 && (errno == EINTR || errno == EAGAIN));
 #else
-            for (int i = 0; i < iovec->iovcnt; i++) {
+            res = 0;
+            for (int i = 0; i < iovcnt; i++) {
                 ssize_t written;
 #ifdef HARD_DEBUG
-		for (int y = 0; y < iovec->iov[i].iov_len; y++)
-                    debug("Write %u\r\n",iovec->iov[i].iov_base[y]);
+		for (int y = 0; y < iov[i].iov_len; y++)
+                    debug("Write %u\r\n",iov[i].iov_base[y]);
 #endif
-                BOOL r = WriteFile(tty->ofd, iovec->iov[i].iov_base,
-                                   iovec->iov[i].iov_len, &written, NULL);
+                BOOL r = WriteFile(tty->ofd, iov[i].iov_base,
+                                   iov[i].iov_len, &written, NULL);
                 if (!r) {
                     res = -1;
                     break;
                 }
                 res += written;
+#ifdef DEBUG
+                /* In debug we simulate a partial write in order to test
+                   the code that handles it */
+                break;
+#endif
             }
 #endif
             if (res < 0) {
