@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 2006-2020. All Rights Reserved.
+ * Copyright Ericsson AB 2006-2024. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -30,6 +30,7 @@
 #include "erl_misc_utils.h"
 
 #if !defined(__WIN32__) /* UNIX */
+#  include <stdarg.h>
 #  include <stdio.h>
 #  include <sys/types.h>
 #  include <sys/param.h>
@@ -40,15 +41,9 @@
 #  ifdef SYS_SELECT_H
 #    include <sys/select.h>
 #  endif
-#  if TIME_WITH_SYS_TIME
-#     include <sys/time.h>
-#     include <time.h>
-#  else
-#     if HAVE_SYS_TIME_H
-#         include <sys/time.h>
-#     else
-#         include <time.h>
-#     endif
+#  include <time.h>
+#  ifdef HAVE_SYS_TIME_H
+#    include <sys/time.h>
 #  endif
 #  include <string.h>
 #  include <stdio.h>
@@ -239,7 +234,7 @@ erts_cpu_info_create(void)
     cpuinfo->online = -1;
     cpuinfo->available = -1;
     cpuinfo->quota = -1;
-    erts_cpu_info_update(cpuinfo);
+    erts_cpu_info_update(cpuinfo, 1);
     return cpuinfo;
 }
 
@@ -264,7 +259,8 @@ erts_cpu_info_destroy(erts_cpu_info_t *cpuinfo)
 }
 
 int
-erts_cpu_info_update(erts_cpu_info_t *cpuinfo)
+erts_cpu_info_update(erts_cpu_info_t *cpuinfo,
+                     int skip_read_topology)
 {
     int changed = 0;
     int configured = 0;
@@ -410,6 +406,8 @@ erts_cpu_info_update(erts_cpu_info_t *cpuinfo)
 
     if (available > online)
 	available = online;
+    else if (available == 0) /* shouldn't happen */
+        available = online;
 
     if (cpuinfo->available != available)
 	changed = 1;
@@ -427,7 +425,9 @@ erts_cpu_info_update(erts_cpu_info_t *cpuinfo)
     old_topology_size = cpuinfo->topology_size;
     cpuinfo->topology = NULL;
 
-    read_topology(cpuinfo);
+    if (!skip_read_topology) {
+        read_topology(cpuinfo);
+    }
 
     if (cpuinfo->topology_size != old_topology_size
 	|| (old_topology_size != 0
@@ -1093,15 +1093,21 @@ get_cgroup_v1_base_dir(const char *controller) {
     return NULL;
 }
 
-static const char*
-get_cgroup_path(const char *controller) {
+enum cgroup_version_t {
+    ERTS_CGROUP_NONE,
+    ERTS_CGROUP_V1,
+    ERTS_CGROUP_V2
+};
+
+static enum cgroup_version_t
+get_cgroup_path(const char *controller, const char **path) {
     char line_buf[10 << 10];
     FILE *var_file;
 
     var_file = fopen("/proc/self/mountinfo", "r");
 
     if (var_file == NULL) {
-        return NULL;
+        return ERTS_CGROUP_NONE;
     }
 
     while (fgets(line_buf, sizeof(line_buf), var_file)) {
@@ -1120,7 +1126,7 @@ get_cgroup_path(const char *controller) {
          * This fails if any of the fs options contain a hyphen, but this is
          * not likely to happen on a cgroup, so we just skip such lines. */
         if (sscanf(line_buf,
-                   "%*d %*d %*d:%*d %4095s %4095s %*s %*[^-]- "
+                   "%*d %*d %*d:%*d %4095s %4095s %*s%*[^-]- "
                    "%63s %*s %511[^\n]\n",
                    root_path, mount_path,
                    fs_type, fs_flags) != 4) {
@@ -1136,7 +1142,9 @@ get_cgroup_path(const char *controller) {
                 if (csv_contains(controllers, controller, ' ')) {
                     free((void*)cgc_path);
                     fclose(var_file);
-                    return strdup(mount_path);
+
+                    *path = strdup(mount_path);
+                    return ERTS_CGROUP_V2;
                 }
             }
             free((void*)cgc_path);
@@ -1145,42 +1153,51 @@ get_cgroup_path(const char *controller) {
                 const char *base_dir = get_cgroup_v1_base_dir(controller);
 
                 if (base_dir) {
-                    const char *result;
-
                     if (strcmp(root_path, base_dir)) {
-                        result = str_combine(mount_path, base_dir);
+                        *path = str_combine(mount_path, base_dir);
                     } else {
-                        result = strdup(mount_path);
+                        *path = strdup(mount_path);
                     }
 
                     free((void*)base_dir);
                     fclose(var_file);
-                    return result;
+
+                    return ERTS_CGROUP_V1;
                 }
             }
         }
     }
 
     fclose(var_file);
-    return NULL;
+
+    return ERTS_CGROUP_NONE;
 }
 
-static int read_cgroup_var(const char *group_path, const char *var_name,
-                           ssize_t *out) {
+static int read_cgroup_interface(const char *group_path, const char *if_name,
+                                 int arg_count, const char *format, ...) {
     const char *var_path;
     int res;
 
-    var_path = str_combine(group_path, var_name);
+    var_path = str_combine(group_path, if_name);
     res = 0;
 
     if (var_path) {
-        FILE *var_file = fopen(var_path, "r");
+        FILE *var_file;
+
+        var_file = fopen(var_path, "r");
         free((void*)var_path);
 
         if (var_file) {
-            if (fscanf(var_file, "%zi", out) == 1) {
+            va_list va_args;
+
+            va_start(va_args, format);
+
+            if (vfscanf(var_file, format, va_args) == arg_count) {
                 res = 1;
             }
+
+            va_end(va_args);
+
             fclose(var_file);
         }
     }
@@ -1195,32 +1212,44 @@ static int read_cgroup_var(const char *group_path, const char *var_name,
 static int
 read_cpu_quota(int limit)
 {
-    const char *cgroup_path = get_cgroup_path("cpu");
+    ssize_t cfs_period_us, cfs_quota_us;
+    const char *cgroup_path;
+    int succeeded;
 
-    if (cgroup_path) {
-        ssize_t cfs_period_us, cfs_quota_us;
-        int succeeded;
-
-        cfs_period_us = -1;
-        cfs_quota_us = -1;
-
-        succeeded =
-            read_cgroup_var(cgroup_path, "/cpu.cfs_quota_us", &cfs_quota_us) &&
-            read_cgroup_var(cgroup_path, "/cpu.cfs_period_us", &cfs_period_us);
+    switch (get_cgroup_path("cpu", &cgroup_path)) {
+    case ERTS_CGROUP_V1:
+        succeeded = read_cgroup_interface(cgroup_path, "/cpu.cfs_quota_us",
+                        1, "%zi", &cfs_quota_us) &&
+                    read_cgroup_interface(cgroup_path, "/cpu.cfs_period_us",
+                        1, "%zi", &cfs_period_us);
 
         free((void*)cgroup_path);
+        break;
+    case ERTS_CGROUP_V2:
+        succeeded = read_cgroup_interface(cgroup_path, "/cpu.max",
+                        2, "%zi %zi", &cfs_quota_us, &cfs_period_us);
 
-        if (succeeded) {
-            if (cfs_period_us > 0 && cfs_quota_us > 0) {
-                size_t quota = cfs_quota_us / cfs_period_us;
+        free((void*)cgroup_path);
+        break;
+    default:
+        succeeded = 0;
+        break;
+    }
 
-                if (quota > 0 && quota <= (size_t)limit) {
-                    return quota;
-                }
+    if (succeeded) {
+        if (cfs_period_us > 0 && cfs_quota_us > 0) {
+            size_t quota = cfs_quota_us / cfs_period_us;
+
+            if (quota == 0) {
+                quota = 1;
             }
 
-            return limit;
+            if (quota > 0 && quota <= (size_t)limit) {
+                return quota;
+            }
         }
+
+        return limit;
     }
 
     return 0;
