@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2000-2023. All Rights Reserved.
+%% Copyright Ericsson AB 2000-2024. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -133,7 +133,12 @@ dialyzer_filter(Undef) ->
 wx_filter(Undef) ->
     case app_exists(wx) of
         false ->
-            filter(fun({_,{MaybeWxModule,_,_}}) ->
+            filter(fun({_,{i, _, _}}) -> false;
+                      ({_,{dbg_iserver, _, _}}) -> false;
+                      ({_,{et_selector, _, _}}) -> false;
+                      ({_,{et_viewer, _, _}}) -> false;
+                      ({_,{int, _, _}}) -> false;
+                      ({_,{MaybeWxModule,_,_}}) ->
                            case atom_to_list(MaybeWxModule) of
                                "wx"++_ -> false;
                                _ -> true
