@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %% 
-%% Copyright Ericsson AB 2007-2023. All Rights Reserved.
+%% Copyright Ericsson AB 2007-2024. All Rights Reserved.
 %% 
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -46,6 +46,7 @@
          shell_history_custom/1, shell_history_custom_errors/1,
 	 job_control_remote_noshell/1,ctrl_keys/1,
          get_columns_and_rows_escript/1,
+         shell_get_password/1,
          shell_navigation/1, shell_multiline_navigation/1, shell_multiline_prompt/1,
          shell_xnfix/1, shell_delete/1,
          shell_transpose/1, shell_search/1, shell_insert/1,
@@ -85,6 +86,7 @@ groups() ->
        job_control_remote, job_control_remote_noshell,
        ctrl_keys, stop_during_init, wrap,
        shell_invalid_ansi,
+       shell_get_password,
        {group, shell_history},
        {group, remsh}]},
      {shell_history, [],
@@ -402,6 +404,8 @@ shell_navigation(Config) ->
              check_location(Term, {0, 0}),
              send_tty(Term,"C-E"),
              check_location(Term, {0, width("{aaa,'b"++U++"b',ccc}")}),
+             send_tty(Term,"C-H"),
+             check_location(Term, {0, width("{aaa,'b"++U++"b',ccc")}),
              send_tty(Term,"C-A"),
              check_location(Term, {0, 0}),
              send_tty(Term,"Enter")
@@ -1192,6 +1196,20 @@ shell_invalid_ansi(_Config) ->
        "-eval","shell:prompt_func({interactive_shell_SUITE,prompt})."
       ]).
 
+shell_get_password(_Config) ->
+   
+    rtnode:run(
+      [{putline,"io:get_password()."},
+       {putline,"secret\r"},
+       {expect, "\r\n\r\n\"secret\""}]),
+
+    %% io:get_password only works when run in "newshell"
+    rtnode:run(
+      [{putline,"io:get_password()."},
+       {expect, "\\Q{error,enotsup}\\E"}],
+     "","",["-oldshell"]).
+
+
 shell_ignore_pager_commands(Config) ->
     Term = start_tty(Config),
     case code:get_doc(file, #{sources=>[eep48]}) of
@@ -1213,12 +1231,19 @@ shell_ignore_pager_commands(Config) ->
 test_valid_keymap(Config) when is_list(Config) ->
     DataDir = proplists:get_value(data_dir,Config),
     Term = setup_tty([{args, ["-config", DataDir ++ "valid_keymap.config"]} | Config]),
+    Prompt = fun() -> ["\e[94m",54620,44397,50612,47,51312,49440,47568,"\e[0m"] end,
+    erpc:call(Term#tmux.node, application, set_env,
+              [stdlib, shell_prompt_func_test,
+               proplists:get_value(shell_prompt_func_test, Config, Prompt)]),
     try
         check_not_in_content(Term, "Invalid key"),
         check_not_in_content(Term, "Invalid function"),
         send_tty(Term, "asdf"),
         send_tty(Term, "C-u"),
         check_content(Term, ">$"),
+        send_tty(Term, "1.\n"),
+        send_tty(Term, "C-b"),
+        check_content(Term, "2>\\s1.$"),
         ok
     after
         stop_tty(Term),
